# Generated by Django 3.0.3 on 2020-07-29 11:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0022_auto_20200728_2051'),
    ]

    operations = [
        migrations.AddField(
            model_name='groupconfig',
            name='fixed_left',
            field=models.TextField(default='', null=True),
        ),
        migrations.AddField(
            model_name='groupconfig',
            name='fixed_right',
            field=models.TextField(default='', null=True),
        ),
    ]
