.right-part{	
	position: absolute;
	left: 20%;
	top: 0;
	height: 100%;
	width: 80%;
}

.right-part .right-content{
	position: absolute;
	left: 0;
	top: 0;
	height: 100%;
	width: 100%;

}

.right-part .group-name{
	position: absolute;
	left: 0;
	top: 2%;
	height: 5%;
	width: 100%;
	overflow: hidden;
	font-size: 16px;
}

.right-part .chart-box{
	position: absolute;
	left: 35%;
	top: 10%;
	height: 85%;
	width: 40%;

	overflow: hidden;
}

.right-part .group-intro{
	position: absolute;
	overflow: hidden;
	left: 5%;
	top: 10%;
	height: 85%;
	width: 25%;

	color: #E0E0E0FF;
	background-color: inherit;
	resize: none;
	overflow: auto;
	font-size: 15px;
}

.right-part .group-intro::placeholder{
	color: #A8A8A8FF;
}
