.editable-input{
	width: 100%;
	left: 0;
	text-align: center;
	color: inherit;
	background-color: inherit;
	height: 100%;
}

/*** 右侧按钮 ***/
.content-a {
	color: #D8D8D8FF !important;
	top: 0;
	height: 100%;
	width: 7%;
	margin-right: 1%;
	margin-left: 1%;
	text-align: center;
}


.content-a:hover , .content-a:hover i {
	background-color: #B2B2B2FF !important;
	color: #333333FF !important;

}
.content-a:active , .content-a:active i{
	background-color: #EAEAEAFF !important;
	color: #000000FF !important;
}
