body{
	overflow : hidden;
}

*{
	border: 0;
	margin: 0;
	padding: 0;
	outline: none;

	color: #D3D3D3FF;	
	cursor: default;

}

/*header 被放到了toolbar里面*/
.header{
	position: absolute;

	background-color: #00000000;
	left: 0;
	top: 0;
	height: 100%;
	width: 50%;
}

/*所以content就是整个页面*/
.content{
	position: absolute;
	left: 0;
	top: 0;
	height: 100%;
	width: 100%;

	background-color: #353535;

	overflow-y: auto;
	overflow-x: hidden !important;
	
}

.hidden{
	display: none;
}

::selection
{
	background : #DDDDDD; 
	color : #383838FF;
}

*{
	font-family: "STXiHei";
}