# Generated by Django 3.0.3 on 2020-04-29 14:18

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0009_auto_20200429_2147'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='variable',
            name='default_val',
        ),
    ]
