<!DOCTYPE html>

{%load static%}

<head>
	<!--导入layui相关样式-->
	<link rel = "stylesheet" href = {% static "/layui/css/layui.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/layui-soul-table/soulTable.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/layer-skin.css" %} type = "text/css" />

	<!--导入layui脚本-->
	<script src = {% static "/layui/layui.js" %}></script>
	<script src = {% static "/layui-soul-table/soulTable.js" %}></script> 
	<script src = {% static "/layui-soul-table/tableFilter.js" %}></script> 
	<script src = {% static "/layui-soul-table/excel.js" %}></script> 
	<script src = {% static "/layui-soul-table/tableChild.js" %}></script>  
	<script src = {% static "/layui-soul-table/tableMerge.js" %}></script> 

	<script>
		var $ = layui.$
	</script>

	<!--导入其他外部库-->
	<script src = "https://cdn.bootcdn.net/ajax/libs/animejs/3.2.0/anime.min.js"></script>
	<script src = "https://cdn.staticfile.org/vue/2.2.2/vue.min.js"></script>

	<!--导入YUI-->
	<link rel = "stylesheet" href = {% static "YUI/css/YUI.css" %} type = "text/css" />
	<script src = "{% static "YUI/YUI.js" %}"></script>
	<script src = "{% static "YUI/YUI-components.js" %}"></script>
	<script src = "{% static "YUI/YUI-css-complement.js" %}"></script>

	<!--针对layui的定制-->
	<link rel = "stylesheet" href = {% static "/experiment/css/group/custom_table_inside.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/group/custom_table_around.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/group/custom_toolbar.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/group/custom_soultable.css" %} type = "text/css" />

	<!--其他样式-->
	<link rel = "stylesheet" href = {% static "/experiment/css/group/group.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/group/header.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/group/content.css" %} type = "text/css" />

	<link rel = "stylesheet" href = {% static "/experiment/css/scrollbar.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/project/layer.css" %} type = "text/css" />
	<link rel = "stylesheet" href = {% static "/experiment/css/utils.css" %} type = "text/css" />

</head>


<script>
	YUI_init()
</script>

<body>

	<!-- 头部部分 -->
	<div class = "header">
		<p class = "group-name">【实验组】{{group.name}}</p>
		<textarea class = "group-intro">{{group.intro}}</textarea>
	</div>

	<!-- 内容（表格）部分 -->
	<div class = "content"><table lay-filter = "main-table" , id = "the-table" , lay-data = "{id:'the-table'}">		
	</table></div>


</body>



<!-- 启用soulTable扩展 -->
<script>
	layui.config({
		base: "{% static "layui-soul-table/"%}"
	}).extend({
		soulTable: "soulTable"  // 模块别名
	})
</script>

<!-- 导入脚本 -->
{% include "experiment/group/scripts.html" %}
<script src = {% static "/experiment/js/utils.js" %}></script>
<script src = {% static "/experiment/js/group/save_and_load.js" %}></script>
<script src = {% static "/experiment/js/group/toolbar_events.js" %}></script>
<script src = {% static "/experiment/js/group/get_table_data.js" %}></script>

<script src = {% static "/experiment/js/group/utils.js" %}></script>
<script src = {% static "/experiment/js/group/group.js" %}></script>
