short_name_len = 50
path_len = 200
value_len = 100

special_postfix = "__FITTERLOG__"
split_token = "__FITTERLOG__SEP__"
config_file_key = "__FITTERLOG__CONFIG__"
cookie_key = "FITTERLOG_PERMISSION"

# 如果为True，会屏蔽登录机制。否则每次reload都要登录。
debugging = True