# Generated by Django 3.0.3 on 2020-05-18 02:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0018_figure'),
    ]

    operations = [
        migrations.AlterField(
            model_name='figure',
            name='name',
            field=models.CharField(max_length=50),
        ),
    ]
