.group-name{
	position : absolute;
	left 	: 0;
	width 	: 100%;
	top 	: 3%;
	height 	: 25%;
	text-align: center;
	line-height: 100%;
}
.group-intro{

	position : absolute;
	left 	: 0;
	width 	: 100%;
	top 	: 30%;
	height 	: 70%;
	text-align : left;
	resize: none;
	background-color: #00000000;
	font-size: 12px;
	color: #D8D8D8FF;
	font-family: "STXiHei";
}


.isDrag{
	background-color: inherit !important;
}
