# Generated by Django 3.0.3 on 2020-05-07 02:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0015_auto_20200507_0929'),
    ]

    operations = [
        migrations.AddField(
            model_name='groupconfig',
            name='hide_bad_exp',
            field=models.BooleanField(default=False),
        ),
    ]
