.header .left-header{
	position: absolute;
	top: 0;
	left: 0;
	height: 100%;
	width: 20%;
}

.header .left-header .project-name{
	position: absolute;
	top: 3%;
	left: 0;
	height: 20px;
	width: 100%;
}
.header .middle-header{
	position: absolute;
	top: 0;
	left: 20%;
	height: 100%;
	width: 50%;
}

.header .middle-header .path-input{
	position: absolute;
	top: 5%;
	left: 5px;
	height: 20%;
	width: 100%;
	border-bottom: 1px solid #DBDBDBFF;

	background-color: inherit;
	resize: none;
}

.header .middle-header .project-intro-input{
	position: absolute;
	top: 30%;
	left: 5px;
	height: 65%;
	width: 100%;

	background-color: inherit;
	resize: none;

	text-align: left;
}

.header .right-header{
	position: absolute;
	top: 0;
	left: 70%;
	height: 100%;
	width: 30%;
}

.header .right-header .new-group-name-input{
	position: absolute;
	top: 5%;
	height : 20%;
	width : 82%;
	left: 5%;
	font-size : 13px;
	background-color : #232329FF;
	outline: none;
}


.header .right-header .new-group-button{
	position: absolute;
	top: 5%;
	width : 5%;
	left : 90%;
	height : 20%;
	cursor : default;
}

.header .right-header .button-box{
	position: absolute;

	top: 35%;
	width : 90%;
	left : 5%;
	bottom : 10%;
}

.header .right-header .button-box .button{
	position: relative;

	margin-left: 	2px;
	margin-right: 	3px;
	margin-top: 	2px;
	margin-bottom: 	3px;
	height: 20px;
	width : 20px;

	cursor : default;
	font-size : 14px;
}

.header .new-group-button,.header .button{
	background-color: inherit;
}
.header .new-group-button:hover,.header .button:hover{
	background-color: #C1C1C1FF;
}