.left-part{
	position: absolute;
	left: 0;
	top: 0;
	height: 100%;
	width: 20%;

	background-color: #202930;
}

.left-part .left-title{
	position: absolute;
	left: 0;
	top: 2%;
	height: 5%;
	width: 100%;
}

.left-part .left-title p{
	position: absolute;
	left: 0;
	top: 0;
	height: 100%;
	width: 100%;
	font-size: 16px;
}

.left-part .left-box{
	position: absolute;
	left: 0;
	top: 7%;
	height: 93%;
	width: 100%;
	overflow-y: auto;
	overflow-x: hidden;
}

.left-part .left-box .group-option
{
	margin-top: 3px;
	margin-left : 0;
	width : 100%;
	line-height: 20px;
	list-style : none;
	transition: 0.5s !important;
	transition-timing-function: ease-out;
	font-size: 16px;

}

.active{
	color : #0F0F0FFF  !important;
	background-color: #B2B2B2FF  !important;
}

