# Generated by Django 3.0.3 on 2020-05-04 08:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0011_auto_20200501_0145'),
    ]

    operations = [
        migrations.AddField(
            model_name='experiment',
            name='end_time',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='experiment',
            name='running',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='experiment',
            name='start_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]
