<!--那些不得不放到html中（因为要使用django语法）的脚本-->

<script>
	var csrf_token = "{{csrf_token}}" //用于提交的密钥

	var save_config_url = "/project/{{project.id}}/save_config" //保存config的地址
	var create_expe_url = "/project/{{project.id}}/create_experiment"
	var hyper_search_url = "/project/{{project.id}}/hyper_search"
</script>


<script>
	/*载入config文件列表*/
	app.config_files = [
		{% for config in config_files %}
			"{{config}}",
		{% endfor %}
	]
	app.config_str = app.config_files.join("\n")

	app.cmd_pref = "{{project.config.cmd_pref}}"
	app.cmd_comm = "{{project.config.cmd_comm}}"
	app.cmd_entr = "{{project.config.cmd_entr}}"
	app.cmd_suff = "{{project.config.cmd_suff}}"
</script>


<script>
	/*添加charts*/
	{% for tot_num , state_num , group in inform%}
		add_chart("chart-container-{{group.id}}" , {{tot_num}} , 
			[{{state_num.0}} , {{state_num.1}} , {{state_num.2}} , {{state_num.3}}]
		)
	{% endfor %}
</script>