# Generated by Django 3.0.3 on 2020-05-18 02:23

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0017_variable_editable'),
    ]

    operations = [
        migrations.CreateModel(
            name='Figure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('html', models.TextField(default='')),
                ('expe', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='figures', to='experiment.Experiment')),
            ],
        ),
    ]
