# Generated by Django 3.0.3 on 2020-04-30 17:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0010_remove_variable_default_val'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hidden_heads', models.TextField(default='')),
                ('hidden_ids', models.TextField(default='')),
            ],
        ),
        migrations.AddField(
            model_name='experimentgroup',
            name='config',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to='experiment.GroupConfig'),
        ),
    ]
