# Generated by Django 3.0.3 on 2020-05-08 16:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0016_groupconfig_hide_bad_exp'),
    ]

    operations = [
        migrations.AddField(
            model_name='variable',
            name='editable',
            field=models.BooleanField(default=False),
        ),
    ]
