.Y-universe{

}

/******************** 位置 ********************/

/*绝对位置*/
.Y-abs-position{
	position: absolute;
}

/*占满父对象*/
.Y-full{
	left: 0;
	top: 0;
	width: 100%;
	height: 100%;
}

/*（容器类）自动横排并排，自动把所有子元素排成一排，尽量占满，子元素需要长度一样*/
.Y-row-separate , .Y-separate-x{
	display: flex;
	flex-direction: row;
	justify-content: space-between;
}

/*（容器类）自动竖排并排，自动把所有子元素排成一列，从上往下依次堆砌，需要子元素自己设置margin*/
.Y-column-stack , .Y-stack-y{
	display: flex;
	flex-direction: column;
	justify-content: flex-start;
}

/*（容器类）自动横排并排，自动把所有子元素排成一列，从上往下依次堆砌，需要子元素自己设置margin*/
.Y-row-stack , .Y-stack-x{
	display: flex;
	flex-direction: row;
	justify-content: flex-start;
}


/******************** 背景颜色设置 ********************/

/*透明背景*/
.Y-color-transparent{
	background-color: #00000000;
}

/*浅色背景*/
.Y-color-light{
	background-color: #D8D8D8;
}

/*深深色背景*/
.Y-color-highdark{
	background-color: #2A2A2A;
}

/*暗深色背景*/
.Y-color-lowdark{
	background-color: #303030;
}

/*深色背景*/
.Y-color-dark{
	background-color: #333333;
}

/*浅深色背景*/
.Y-color-lightdark{
	background-color: #4B4B4B;
}

/******************** 文字颜色设置 ********************/

/*浅色文字*/
.Y-color-text-light{
	color: #D8D8D8FF;
}

.Y-color-text-light::selection
{
	background : #AAAAAAFF; 
	color : #BBBBBBFF;
}

/*深色文字*/
.Y-color-text-dark{
	color: #333333;
}
.Y-color-text-dark::selection
{
	background : #2D2D2DFF; 
	color : #D6D6D6FF;
}

/******************** 字体设置 ********************/

/*标题字体*/
.Y-font-title{
	font-family : "DengXian";
}

/*不正式标题字体*/
.Y-font-shorttitle{
	font-family : "STXihei";
}

/*短文本字体*/
.Y-font-short{
	font-family : "STXihei";
}

/*正文字体*/
.Y-font-default{
	font-family : "FangSong";
}

/******************** 滚动条选项 ********************/

/*隐藏滚动条，禁止滚动*/
.Y-hide-overflow{
	overflow: hidden;
}
.Y-hide-overflow-x{
	overflow-x: hidden;
}
.Y-no-scrollbar::-webkit-scrollbar{
	display: none;
}

/*使用滚动条*/
.Y-scroll{
	overflow: auto;
}
.Y-scroll-y{
	overflow-y: auto;
}

/*隐藏滚动条，可以滚动*/
.Y-allow-overflow{
	overflow: visible;
}

/******************** 文字效果 ********************/

/*文字只竖直居中*/
.Y-text-vertical-center , .Y-text-center-y{
	/*还有一个垂直居中的效果在js中实现*/
	/*注意这种效果是不能嵌套的*/
}

/*文字居中*/
.Y-text-center{
	text-align: center;
	/*还有一个垂直居中的效果在js中实现*/
}

/******************** 其他 ********************/

/*不可见*/
.Y-unshow{
	display: none;
}
