
/*** 修改设置文件列表 ***/
.config-name-input{
	position: absolute;
	top: 15%;
	left: 5%;
	width: 80%;
	height: 71%;

	font-size: 15px;

	color: #E5E5E5FF;
	background-color: #333333FF;
	resize: none;

	overflow: auto;
}

.config-layer .my-close{
	top: 50px  !important;
}

/*** 超参数搜索 ***/
.search-name-input{
	position: absolute;
	color: #E5E5E5FF;
	left: 5%;
	top: 20px;
	width: 70%;
	height: 16px;
	line-height: 16px;
	padding: 0;
	margin: 0;
}

.search-button{
	position: absolute;
	color: #E5E5E5FF;
	background-color: inherit;
	left: 82%;
	top: 20px;
	width: 16px;
	height: 16px;
	line-height: 16px;
}

.search-button:hover{
	color: #333333FF !important;
	background-color: #E5E5E5FF !important;
}

/*** 设置默认命令 ***/
.set-cmd-layer .set-cmd-input{
	position: relative;
	color: #E5E5E5FF;
	left: 5%;
	margin-top: 5px !important;
	width: 70%;
	height: 16px;
	line-height: 16px;
	padding: 0;
}
