# Generated by Django 3.0.3 on 2020-04-27 17:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0004_experiment_start_time'),
    ]

    operations = [
        migrations.AlterField(
            model_name='experimentgroup',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='groups', to='experiment.Project', unique=True),
        ),
    ]
