.my-skin *{
	margin: 0 !important;
	padding: 0 !important;
}

.my-skin{
	opacity: 90%;
	background-color: #00000000 !important; 
}

.my-skin .layui-layer-title{
	background-color:	#333333; 
	color:				#E5E5E5FF; 
	border: 			none;
	height: 			30px;
	width:				100%;
	display: 			flex;
	align-items: center;
	justify-content: center;
}
.my-skin .layui-layer-title .title-text{
	text-align: center;
}
/*内容*/
.my-skin .layui-layer-content{
	background-color:	#4F4F4F; 
	color:				#E5E5E5FF; 
	border: 			none;
}
.my-skin .layui-layer-btn{
	display: none;
}
.my-skin .layui-layer-setwin{
	display: none;
}

.my-skin .my-close{
	position: absolute;
	right: 5%;
	top: 30%;
	width: 16px;
	height: 16px;
	line-height: 16px;
	background-color:	#4F4F4F !important; 
	color:				#E5E5E5FF !important; 
}

.my-skin .my-close:hover{
	background-color:	#E5E5E5FF !important; 
	color:				#4F4F4F !important; 
}