body{
	overflow : hidden;
}

*{
	border: 0;
	margin: 0;
	padding: 0;
	outline: none;

	color: #D3D3D3FF;
	cursor: default;
}

.header{
	position: absolute;

	background-color: #202930;
	left: 0;
	top: 0;
	height: 15%;
	width: 100%;
}

.content{
	position: absolute;
	left: 0;
	top: 15%;
	height: 85%;
	width: 100%;

	background-color: #131E27;
}

.hidden{
	display: none;
}

::selection
{
	background : #DDDDDD; 
	color : #383838FF;
}

p , input , textarea{
	font-family: "DengXian";
}

::-webkit-scrollbar-thumb:hover,
::-webkit-scrollbar-thumb:vertical:hover {
	background-color: #5E75AA  !important;
}