{% load static %}

<head>
	<!-- Layui -->
	<link rel = "stylesheet" href = {% static "/layui/css/layui.css" %} type = "text/css" />
	<script src = {% static "/layui/layui.js" %}></script>

	<!-- highcharts -->
	<script src = "{% static "Highcharts/highcharts.src.js" %} "></script>
	<script src = "{% static "Highcharts/themes/dark-unica.js" %} "></script>

	<!-- 样式 -->
	<link rel = "stylesheet" href = {% static "/experiment/css/variable/variable.css" %} type = "text/css" />

</head>

<body>

<div class = "header">
	<div class = "decs">
		<p> 【Experiment id】 {{variable.expe.id}}</p>
		<p> 【Variable】 {{variable.name}}</p>
	</div>

	<div class = "buts">
		<a class = "layui-icon" href = "/group/{{variable.expe.group.id}}" title = "返回"> &#xe65c;</a>
	</div>
</div>

<div class = "layui-tab">
  <ul class = "layui-tab-title">

  	<li class = "layui-this tab-head">ALL</li>
	{% for tra , lis in tracks_and_values%}
		<li class = "tab-head">{{tra.name}}</li>
	{% endfor %}
  </ul>
  <div class = "layui-tab-content">

  	<div id = "all-container" class = "layui-show layui-tab-item"></div>

  	{% for tra , lis in tracks_and_values %}
		<div class = "layui-tab-item">
			<div id = "container-{{tra.id}}"></div>
		</div>
	{% endfor %}
  </div>
</div>

</body>


<!-- 启用layui的tab栏 -->
<script>
	layui.use("element", function(){});
</script>

<!-- 脚本 -->
<script src = "{% static "experiment/js/variable/charts.js" %} "></script>
{% include "./scripts.html" %}
