# Generated by Django 3.0.3 on 2020-05-07 01:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0014_groupconfig_show_order'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='experiment',
            name='running',
        ),
        migrations.AddField(
            model_name='experiment',
            name='state',
            field=models.IntegerField(default=0),
        ),
    ]
