# Generated by Django 3.0.3 on 2020-04-29 04:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0006_auto_20200428_0141'),
    ]

    operations = [
        migrations.AddField(
            model_name='experiment',
            name='intro',
            field=models.TextField(default=''),
        ),
        migrations.AddField(
            model_name='experimentgroup',
            name='intro',
            field=models.TextField(default=''),
        ),
        migrations.AddField(
            model_name='project',
            name='intro',
            field=models.TextField(default=''),
        ),
    ]
