# Generated by Django 3.0.3 on 2020-04-29 13:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0008_variable_default_val'),
    ]

    operations = [
        migrations.AlterField(
            model_name='singlevalue',
            name='value',
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name='variable',
            name='default_val',
            field=models.CharField(default='', max_length=100),
        ),
    ]
