# Generated by Django 3.0.3 on 2020-04-26 07:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('experiment', '0003_auto_20200426_1535'),
    ]

    operations = [
        migrations.AddField(
            model_name='experiment',
            name='start_time',
            field=models.DateField(auto_now=True),
        ),
    ]
