/*以custom开头的文件：对框架元素的重新定制*/

.layui-table-tool{
	background-color: #353535 !important;
	padding: 0 !important;
	margin: 0 !important;
	outline: 0 !important;
	top: 0 !important;
	height: 8% !important;
	left: 0 !important;
	width: 100% !important;
}

/*导出的弹出框*/
.layui-table-tool-panel{
	color : #DBDBDBFF;
	background-color: #25656E !important;
	padding: 0 !important;
	margin: 0 !important;
	outline: 0 !important;
	border: solid 1px !important;
	border-color: #DBDBDBFF !important;

	transition: 0.5s all !important;

}

.layui-table-tool-panel li .layui-form-checkbox span {
	color: #DBDBDBFF !important;
	background-color: #25656E !important;
	transition: 0.5s all !important;
}

.layui-table-tool-panel li:hover{
	color : #25656E !important;
	background-color: #DBDBDBFF !important;
	transition-timing-function: ease-out !important;
	transition: 1s all !important;
}

.layui-table-tool-panel li:hover .layui-form-checkbox span{
	color : #25656E !important;
	background-color: #DBDBDBFF !important;

	transition-timing-function: ease-out !important;
	transition: 1s all !important;
}

/*工具栏按钮*/
.layui-table-tool-self .layui-inline {
	color: #DBDBDBFF !important;
	border: 1px solid #DBDBDBFF !important;
	background-color: #353535 !important;
	transition: none !important;

}

.layui-table-tool-self .layui-inline i{
	color: #DBDBDBFF !important;
}

.inverse-color{
	filter: invert(100%);
}

/*取消动画*/
.layui-table-tool-self .layui-inline:hover {
	color: #4C4C4CFF !important;
	border: 1px solid #00000000 !important;
	background-color: #DBDBDBFF !important;
	transition: none !important;
}

.layui-table-tool-self .layui-inline:hover i{
	color: #4C4C4CFF !important;
	transition: none !important;
}


/*弹出框的复选框*/
.layui-table-tool-panel .layui-form-checkbox[lay-skin=primary] i{
	background-color: #3796A3 !important;
	color: #3796A3 !important;

	border: none !important;
	transition: none !important;
	transition: 0.5s !important;
	transition-timing-function: ease-out !important;
}

.layui-table-tool-panel li:hover .layui-form-checkbox[lay-skin=primary] i{
	background-color: #20595E !important;
	color: #20595E !important;
	border: none !important;
	transition: none !important;
	transition: 1s !important;
	transition-timing-function: ease-out !important;
}

.layui-table-tool-panel .layui-form-checked[lay-skin=primary] i{
	background-color: #1B494F !important;
	color: #C1CFE6 !important;
	font-weight: bold !important;
	border: none !important;
	transition: 0.5s !important;
	transition-timing-function: ease-out !important;
}
.layui-table-tool-panel li:hover .layui-form-checked[lay-skin=primary] i{
	background-color: #2E7E88 !important;
	color: #0F2A2D !important;
	font-weight: bold !important;
	border: none !important;
	transition: 1s !important;
	transition-timing-function: ease-out !important;
}
